@@ -314,12 +314,12 @@ public abstract class ImageReaderBase extends ImageReader {
 
         long dimension = (long) destWidth * destHeight;
         if (dimension > Integer.MAX_VALUE) {
-            throw new IllegalArgumentException(String.format("destination width * height > Integer.MAX_VALUE: %d", dimension));
+            throw new IIOException(String.format("destination width * height > Integer.MAX_VALUE: %d", dimension));
         }
 
         long size = dimension * imageType.getSampleModel().getNumDataElements();
         if (size > Integer.MAX_VALUE) {
-            throw new IllegalArgumentException(String.format("destination width * height * samplesPerPixel > Integer.MAX_VALUE: %d", size));
+            throw new IIOException(String.format("destination width * height * samplesPerPixel > Integer.MAX_VALUE: %d", size));
         }
 
         // Create a new image based on the type specifier
@@ -440,6 +440,7 @@ public abstract class ImageReaderBase extends ImageReader {
         static final String ZOOM_IN = "zoom-in";
         static final String ZOOM_OUT = "zoom-out";
         static final String ZOOM_ACTUAL = "zoom-actual";
+        static final String ZOOM_FIT = "zoom-fit";
 
         private BufferedImage image;
 
@@ -515,9 +516,20 @@ public abstract class ImageReaderBase extends ImageReader {
 
         private void setupActions() {
             // Mac weirdness... VK_MINUS/VK_PLUS seems to map to english key map always...
-            bindAction(new ZoomAction("Zoom in", 2), ZOOM_IN, KeyStroke.getKeyStroke('+'), KeyStroke.getKeyStroke(KeyEvent.VK_ADD, 0));
-            bindAction(new ZoomAction("Zoom out", .5), ZOOM_OUT, KeyStroke.getKeyStroke('-'), KeyStroke.getKeyStroke(KeyEvent.VK_SUBTRACT, 0));
-            bindAction(new ZoomAction("Zoom actual"), ZOOM_ACTUAL, KeyStroke.getKeyStroke('0'), KeyStroke.getKeyStroke(KeyEvent.VK_0, 0));
+            bindAction(new ZoomAction("Zoom in", 2), ZOOM_IN,
+                    KeyStroke.getKeyStroke('+'),
+                    KeyStroke.getKeyStroke(KeyEvent.VK_PLUS, Toolkit.getDefaultToolkit().getMenuShortcutKeyMask()),
+                    KeyStroke.getKeyStroke(KeyEvent.VK_ADD, Toolkit.getDefaultToolkit().getMenuShortcutKeyMask()));
+            bindAction(new ZoomAction("Zoom out", .5), ZOOM_OUT,
+                    KeyStroke.getKeyStroke('-'),
+                    KeyStroke.getKeyStroke(KeyEvent.VK_MINUS, Toolkit.getDefaultToolkit().getMenuShortcutKeyMask()),
+                    KeyStroke.getKeyStroke(KeyEvent.VK_SUBTRACT, Toolkit.getDefaultToolkit().getMenuShortcutKeyMask()));
+            bindAction(new ZoomAction("Zoom actual"), ZOOM_ACTUAL,
+                    KeyStroke.getKeyStroke('0'),
+                    KeyStroke.getKeyStroke(KeyEvent.VK_0, Toolkit.getDefaultToolkit().getMenuShortcutKeyMask()));
+            bindAction(new ZoomToFitAction("Zoom fit"), ZOOM_FIT,
+                    KeyStroke.getKeyStroke('9'),
+                    KeyStroke.getKeyStroke(KeyEvent.VK_9, Toolkit.getDefaultToolkit().getMenuShortcutKeyMask()));
 
             bindAction(TransferHandler.getCopyAction(), (String) TransferHandler.getCopyAction().getValue(Action.NAME), KeyStroke.getKeyStroke(KeyEvent.VK_C, Toolkit.getDefaultToolkit().getMenuShortcutKeyMask()));
             bindAction(TransferHandler.getPasteAction(), (String) TransferHandler.getPasteAction().getValue(Action.NAME), KeyStroke.getKeyStroke(KeyEvent.VK_V, Toolkit.getDefaultToolkit().getMenuShortcutKeyMask()));
@@ -534,6 +546,7 @@ public abstract class ImageReaderBase extends ImageReader {
         private JPopupMenu createPopupMenu() {
             JPopupMenu popup = new JPopupMenu();
 
+            popup.add(getActionMap().get(ZOOM_FIT));
             popup.add(getActionMap().get(ZOOM_ACTUAL));
             popup.add(getActionMap().get(ZOOM_IN));
             popup.add(getActionMap().get(ZOOM_OUT));
@@ -554,7 +567,7 @@ public abstract class ImageReaderBase extends ImageReader {
             addCheckBoxItem(new ChangeBackgroundAction("Dark", Color.DARK_GRAY), background, group);
             addCheckBoxItem(new ChangeBackgroundAction("Black", Color.BLACK), background, group);
             background.addSeparator();
-            ChooseBackgroundAction chooseBackgroundAction = new ChooseBackgroundAction("Choose...", defaultBG != null ? defaultBG : Color.BLUE);
+            ChooseBackgroundAction chooseBackgroundAction = new ChooseBackgroundAction("Choose...", defaultBG != null ? defaultBG : new Color(0xFF6600));
             chooseBackgroundAction.putValue(Action.SELECTED_KEY, backgroundPaint == defaultBG);
             addCheckBoxItem(chooseBackgroundAction, background, group);
 
@@ -668,14 +681,41 @@ public abstract class ImageReaderBase extends ImageReader {
                 }
                 else {
                     Icon current = getIcon();
-                    int w = (int) Math.max(Math.min(current.getIconWidth() * zoomFactor, image.getWidth() * 16), image.getWidth() / 16);
-                    int h = (int) Math.max(Math.min(current.getIconHeight() * zoomFactor, image.getHeight() * 16), image.getHeight() / 16);
+                    int w = Math.max(Math.min((int) (current.getIconWidth() * zoomFactor), image.getWidth() * 16), image.getWidth() / 16);
+                    int h = Math.max(Math.min((int) (current.getIconHeight() * zoomFactor), image.getHeight() * 16), image.getHeight() / 16);
 
                     setIcon(new BufferedImageIcon(image, Math.max(w, 2), Math.max(h, 2), w > image.getWidth() || h > image.getHeight()));
                 }
             }
         }
 
+        private class ZoomToFitAction extends ZoomAction {
+            public ZoomToFitAction(final String name) {
+                super(name, -1);
+            }
+
+            public void actionPerformed(final ActionEvent e) {
+                JComponent source = (JComponent) e.getSource();
+
+                if (source instanceof JMenuItem) {
+                    JPopupMenu menu = (JPopupMenu) SwingUtilities.getAncestorOfClass(JPopupMenu.class, source);
+                    source = (JComponent) menu.getInvoker();
+                }
+
+                Container container = SwingUtilities.getAncestorOfClass(JViewport.class, source);
+
+                double ratioX = container.getWidth() / (double) image.getWidth();
+                double ratioY = container.getHeight() / (double) image.getHeight();
+
+                double zoomFactor = Math.min(ratioX, ratioY);
+
+                int w = Math.max(Math.min((int) (image.getWidth() * zoomFactor), image.getWidth() * 16), image.getWidth() / 16);
+                int h = Math.max(Math.min((int) (image.getHeight() * zoomFactor), image.getHeight() * 16), image.getHeight() / 16);
+
+                setIcon(new BufferedImageIcon(image, w, h, zoomFactor > 1));
+            }
+        }
+
         private static class ImageTransferable implements Transferable {
             private final BufferedImage image;
 
@@ -694,7 +734,7 @@ public abstract class ImageReaderBase extends ImageReader {
             }
 
             @Override
-            public Object getTransferData(final DataFlavor flavor) throws UnsupportedFlavorException, IOException {
+            public Object getTransferData(final DataFlavor flavor) throws UnsupportedFlavorException {
                 if (isDataFlavorSupported(flavor)) {
                     return image;
                 }
