@@ -57,8 +57,10 @@ public final class BufferedImageInputStream extends ImageInputStreamImpl impleme
 
     private ImageInputStream stream;
 
-    private ByteBuffer buffer;
-    private ByteBuffer integralCache = ByteBuffer.allocate(8);
+    private  ByteBuffer buffer;
+
+    private final ByteBuffer integralCache = ByteBuffer.allocate(8);
+    private final byte[] integralCacheArray = integralCache.array();
 
     public BufferedImageInputStream(final ImageInputStream pStream) throws IOException {
         this(pStream, DEFAULT_BUFFER_SIZE);
@@ -97,10 +99,10 @@ public final class BufferedImageInputStream extends ImageInputStreamImpl impleme
 
         if (!buffer.hasRemaining()) {
             fillBuffer();
-        }
 
-        if (!buffer.hasRemaining()) {
-            return -1;
+            if (!buffer.hasRemaining()) {
+                return -1;
+            }
         }
 
         bitOffset = 0;
@@ -172,21 +174,21 @@ public final class BufferedImageInputStream extends ImageInputStreamImpl impleme
 
     @Override
     public short readShort() throws IOException {
-        readFully(integralCache.array(), 0, 2);
+        readFully(integralCacheArray, 0, 2);
 
         return integralCache.getShort(0);
     }
 
     @Override
     public int readInt() throws IOException {
-        readFully(integralCache.array(), 0, 4);
+        readFully(integralCacheArray, 0, 4);
 
         return integralCache.getInt(0);
     }
 
     @Override
     public long readLong() throws IOException {
-        readFully(integralCache.array(), 0, 8);
+        readFully(integralCacheArray, 0, 8);
 
         return integralCache.getLong(0);
     }
@@ -253,6 +255,7 @@ public final class BufferedImageInputStream extends ImageInputStreamImpl impleme
             }
 
             int val = buffer.get() & 0xff;
+            streamPos++;
 
             accum <<= 8;
             accum |= val;
@@ -262,9 +265,7 @@ public final class BufferedImageInputStream extends ImageInputStreamImpl impleme
         // Move byte position back if in the middle of a byte
         if (newBitOffset != 0) {
             buffer.position(buffer.position() - 1);
-        }
-        else {
-            streamPos++;
+            streamPos--;
         }
 
         this.bitOffset = newBitOffset;
@@ -279,26 +280,26 @@ public final class BufferedImageInputStream extends ImageInputStreamImpl impleme
     }
 
     @Override
-    public void seek(long pPosition) throws IOException {
+    public void seek(long position) throws IOException {
         checkClosed();
         bitOffset = 0;
 
-        if (streamPos == pPosition) {
+        if (streamPos == position) {
             return;
         }
 
         // Optimized to not invalidate buffer if new position is within current buffer
-        long newBufferPos = buffer.position() + pPosition - streamPos;
+        long newBufferPos = buffer.position() + position - streamPos;
         if (newBufferPos >= 0 && newBufferPos <= buffer.limit()) {
             buffer.position((int) newBufferPos);
         }
         else {
             // Will invalidate buffer
             buffer.limit(0);
-            stream.seek(pPosition);
+            stream.seek(position);
         }
 
-        streamPos = pPosition;
+        streamPos = position;
     }
 
     @Override
@@ -330,7 +331,9 @@ public final class BufferedImageInputStream extends ImageInputStreamImpl impleme
     @Override
     public void close() throws IOException {
         if (stream != null) {
-            //stream.close();
+            // TODO: FixMe: Need to close underlying stream here!
+            //   For call sites that relies on not closing, we should instead not close the buffered stream.
+//             stream.close();
             stream = null;
             buffer = null;
         }
