@@ -41,7 +41,6 @@ import com.twelvemonkeys.imageio.metadata.jpeg.JPEGSegment;
 import com.twelvemonkeys.imageio.metadata.jpeg.JPEGSegmentUtil;
 import com.twelvemonkeys.imageio.metadata.tiff.TIFF;
 import com.twelvemonkeys.imageio.metadata.tiff.TIFFReader;
-import com.twelvemonkeys.imageio.stream.BufferedImageInputStream;
 import com.twelvemonkeys.imageio.stream.ByteArrayImageInputStream;
 import com.twelvemonkeys.imageio.stream.SubImageInputStream;
 import com.twelvemonkeys.imageio.util.ImageTypeSpecifiers;
@@ -57,13 +56,13 @@ import javax.imageio.metadata.IIOMetadataFormatImpl;
 import javax.imageio.metadata.IIOMetadataNode;
 import javax.imageio.spi.ImageReaderSpi;
 import javax.imageio.stream.ImageInputStream;
-import javax.imageio.stream.MemoryCacheImageInputStream;
 import java.awt.*;
 import java.awt.color.ColorSpace;
 import java.awt.color.ICC_ColorSpace;
 import java.awt.color.ICC_Profile;
 import java.awt.image.*;
 import java.io.*;
+import java.nio.ByteOrder;
 import java.util.List;
 import java.util.*;
 
@@ -113,9 +112,6 @@ public final class JPEGImageReader extends ImageReaderBase {
     /** Internal constant for referring all APP segments */
     static final int ALL_APP_MARKERS = -1;
 
-    /** Segment identifiers for the JPEG segments we care about reading. */
-    private static final Map<Integer, List<String>> SEGMENT_IDENTIFIERS = JPEGSegmentUtil.ALL_SEGMENTS;
-
     /** Our JPEG reading delegate */
     private final ImageReader delegate;
 
@@ -126,13 +122,11 @@ public final class JPEGImageReader extends ImageReaderBase {
     private ImageReader thumbnailReader;
     private List<ThumbnailReader> thumbnails;
 
-    private JPEGImage10MetadataCleaner metadataCleaner;
-
     /** Cached list of JPEG segments we filter from the underlying stream */
     private List<Segment> segments;
 
     private int currentStreamIndex = 0;
-    private List<Long> streamOffsets = new ArrayList<>();
+    private final List<Long> streamOffsets = new ArrayList<>();
 
     protected JPEGImageReader(final ImageReaderSpi provider, final ImageReader delegate) {
         super(provider);
@@ -161,8 +155,6 @@ public final class JPEGImageReader extends ImageReaderBase {
             thumbnailReader.reset();
         }
 
-        metadataCleaner = null;
-
         installListeners();
     }
 
@@ -191,10 +183,10 @@ public final class JPEGImageReader extends ImageReaderBase {
                 return true;
             }
         }
-        catch (IIOException ignore) {
+        catch (IIOException e) {
             // May happen if no SOF is found, in case we'll just fall through
             if (DEBUG) {
-                ignore.printStackTrace();
+                e.printStackTrace();
             }
         }
 
@@ -472,15 +464,12 @@ public final class JPEGImageReader extends ImageReaderBase {
         }
 
         // We'll need a read param
-        Rectangle origSourceRegion;
         if (param == null) {
             param = delegate.getDefaultReadParam();
-            origSourceRegion = null;
-        }
-        else {
-            origSourceRegion = param.getSourceRegion();
         }
 
+        Rectangle origSourceRegion = param.getSourceRegion();
+
         Rectangle srcRegion = new Rectangle();
         Rectangle dstRegion = new Rectangle();
         computeRegions(param, origWidth, origHeight, image, srcRegion, dstRegion);
@@ -534,7 +523,7 @@ public final class JPEGImageReader extends ImageReaderBase {
         return image;
     }
 
-    private JPEGColorSpace getSourceCSType(final JFIF jfif, final AdobeDCT adobeDCT, final Frame startOfFrame) throws IIOException {
+    static JPEGColorSpace getSourceCSType(final JFIF jfif, final AdobeDCT adobeDCT, final Frame startOfFrame) throws IIOException {
         // Adapted from libjpeg jdapimin.c:
         // Guess the input colorspace
         // (Wish JPEG committee had provided a real way to specify this...)
@@ -717,10 +706,11 @@ public final class JPEGImageReader extends ImageReaderBase {
 
     private void initHeader(final int imageIndex) throws IOException {
         if (imageIndex < 0) {
-            throw new IllegalArgumentException("imageIndex < 0: " + imageIndex);
+            throw new IndexOutOfBoundsException("imageIndex < 0: " + imageIndex);
         }
 
         if (imageIndex == currentStreamIndex) {
+            initHeader();
             return;
         }
 
@@ -743,26 +733,26 @@ public final class JPEGImageReader extends ImageReaderBase {
             long lastKnownSOIOffset = streamOffsets.get(streamOffsets.size() - 1);
             imageInput.seek(lastKnownSOIOffset);
 
-            try (ImageInputStream stream = new BufferedImageInputStream(imageInput)) { // Extreme (10s -> 50ms) speedup if imageInput is FileIIS
+            try {
                 for (int i = streamOffsets.size() - 1; i < imageIndex; i++) {
                     long start = 0;
 
                     if (DEBUG) {
                         start = System.currentTimeMillis();
-                        System.out.println(String.format("Start seeking for image index %d", i + 1));
+                        System.out.printf("Start seeking for image index %d%n", i + 1);
                     }
 
                     // Need to skip over segments, as they may contain JPEG markers (eg. JFXX or EXIF thumbnail)
-                    JPEGSegmentUtil.readSegments(stream, Collections.<Integer, List<String>>emptyMap());
+                    JPEGSegmentUtil.readSegments(imageInput, Collections.<Integer, List<String>>emptyMap());
 
                     // Now, search for EOI and following SOI...
                     int marker;
-                    while ((marker = stream.read()) != -1) {
-                        if (marker == 0xFF && (0xFF00 | stream.readUnsignedByte()) == JPEG.EOI) {
+                    while ((marker = imageInput.read()) != -1) {
+                        if (marker == 0xFF && (0xFF00 | imageInput.readUnsignedByte()) == JPEG.EOI) {
                             // Found EOI, now the SOI should be nearby...
-                            while ((marker = stream.read()) != -1) {
-                                if (marker == 0xFF && (0xFF00 | stream.readUnsignedByte()) == JPEG.SOI) {
-                                    long nextSOIOffset = stream.getStreamPosition() - 2;
+                            while ((marker = imageInput.read()) != -1) {
+                                if (marker == 0xFF && (0xFF00 | imageInput.readUnsignedByte()) == JPEG.SOI) {
+                                    long nextSOIOffset = imageInput.getStreamPosition() - 2;
                                     imageInput.seek(nextSOIOffset);
                                     streamOffsets.add(nextSOIOffset);
 
@@ -776,10 +766,9 @@ public final class JPEGImageReader extends ImageReaderBase {
                     }
 
                     if (DEBUG) {
-                        System.out.println(String.format("Seek in %d ms", System.currentTimeMillis() - start));
+                        System.out.printf("Seek in %d ms%n", System.currentTimeMillis() - start);
                     }
                 }
-
             }
             catch (EOFException eof) {
                 IndexOutOfBoundsException ioobe = new IndexOutOfBoundsException("Image index " + imageIndex + " not found in stream");
@@ -837,11 +826,11 @@ public final class JPEGImageReader extends ImageReaderBase {
         try {
             imageInput.seek(streamOffsets.get(currentStreamIndex));
 
-            return JPEGSegmentUtil.readSegments(imageInput, SEGMENT_IDENTIFIERS);
+            return JPEGSegmentUtil.readSegments(imageInput, JPEGSegmentUtil.ALL_SEGMENTS);
         }
-        catch (IIOException | IllegalArgumentException ignore) {
+        catch (IIOException | IllegalArgumentException e) {
             if (DEBUG) {
-                ignore.printStackTrace();
+                e.printStackTrace();
             }
         }
         finally {
@@ -905,16 +894,19 @@ public final class JPEGImageReader extends ImageReaderBase {
 
         if (!exifSegments.isEmpty()) {
             Application exif = exifSegments.get(0);
-            InputStream data = exif.data();
+            int offset = exif.identifier.length() + 2; // Incl. pad
 
-            if (data.read() == -1) { // Read pad
+            if (exif.data.length <= offset) {
                 processWarningOccurred("Exif chunk has no data.");
             }
             else {
-                ImageInputStream stream = new MemoryCacheImageInputStream(data);
-                return (CompoundDirectory) new TIFFReader().read(stream);
-
-                // TODO: Directory offset of thumbnail is wrong/relative to container stream, causing trouble for the TIFFReader...
+                // TODO: Consider returning ByteArrayImageInputStream from Segment.data()
+                try (ImageInputStream stream = new ByteArrayImageInputStream(exif.data, offset, exif.data.length - offset)) {
+                    return (CompoundDirectory) new TIFFReader().read(stream);
+                }
+                catch (IIOException e) {
+                    processWarningOccurred("Exif chunk is present, but can't be read: " + e.getMessage());
+                }
             }
         }
 
@@ -1102,6 +1094,7 @@ public final class JPEGImageReader extends ImageReaderBase {
             // Read JFIF thumbnails if present
             JFIF jfif = getJFIF();
             if (jfif != null && jfif.thumbnail != null) {
+                // TODO: Check if the JFIF segment really has room for this thumbnail?
                 thumbnails.add(new JFIFThumbnailReader(thumbnailProgressDelegator, imageIndex, thumbnails.size(), jfif));
             }
 
@@ -1112,6 +1105,7 @@ public final class JPEGImageReader extends ImageReaderBase {
                     case JFXX.JPEG:
                     case JFXX.INDEXED:
                     case JFXX.RGB:
+                        // TODO: Check if the JFXX segment really has room for this thumbnail?
                         thumbnails.add(new JFXXThumbnailReader(thumbnailProgressDelegator, getThumbnailReader(), imageIndex, thumbnails.size(), jfxx));
                         break;
                     default:
@@ -1125,48 +1119,77 @@ public final class JPEGImageReader extends ImageReaderBase {
                 Application exif = exifSegments.get(0);
 
                 // Identifier is "Exif\0" + 1 byte pad
-                int offset = exif.identifier.length() + 2;
+                int dataOffset = exif.identifier.length() + 2;
 
-                if (exif.data.length <= offset) {
+                if (exif.data.length <= dataOffset) {
                     processWarningOccurred("Exif chunk has no data.");
                 }
                 else {
-                    ImageInputStream stream = new ByteArrayImageInputStream(exif.data, offset, exif.data.length - offset);
-                    CompoundDirectory exifMetadata = (CompoundDirectory) new TIFFReader().read(stream);
+                    ImageInputStream stream = new ByteArrayImageInputStream(exif.data, dataOffset, exif.data.length - dataOffset);
+                    try {
+                        CompoundDirectory exifMetadata = (CompoundDirectory) new TIFFReader().read(stream);
 
-                    if (exifMetadata.directoryCount() == 2) {
-                        Directory ifd1 = exifMetadata.getDirectory(1);
+                        if (exifMetadata.directoryCount() == 2) {
+                            Directory ifd1 = exifMetadata.getDirectory(1);
 
-                        // Compression: 1 = no compression, 6 = JPEG compression (default)
-                        Entry compressionEntry = ifd1.getEntryById(TIFF.TAG_COMPRESSION);
-                        int compression = compressionEntry == null ? 6 : ((Number) compressionEntry.getValue()).intValue();
+                            // Compression: 1 = no compression, 6 = JPEG compression (default)
+                            Entry compressionEntry = ifd1.getEntryById(TIFF.TAG_COMPRESSION);
+                            int compression = compressionEntry == null ? 6 : ((Number) compressionEntry.getValue()).intValue();
 
-                        if (compression == 6) {
-                            if (ifd1.getEntryById(TIFF.TAG_JPEG_INTERCHANGE_FORMAT) != null) {
-                                Entry jpegLength = ifd1.getEntryById(TIFF.TAG_JPEG_INTERCHANGE_FORMAT_LENGTH);
+                            if (compression == 6) {
+                                Entry jpegOffEntry = ifd1.getEntryById(TIFF.TAG_JPEG_INTERCHANGE_FORMAT);
+                                if (jpegOffEntry != null) {
+                                    Entry jpegLenEntry = ifd1.getEntryById(TIFF.TAG_JPEG_INTERCHANGE_FORMAT_LENGTH);
 
-                                if ((jpegLength == null || ((Number) jpegLength.getValue()).longValue() > 0)) {
-                                    thumbnails.add(new EXIFThumbnailReader(thumbnailProgressDelegator, getThumbnailReader(), 0, thumbnails.size(), ifd1, stream));
+                                    // Test if Exif thumbnail is contained within the Exif segment (offset + length <= segment.length)
+                                    long jpegOffset = ((Number) jpegOffEntry.getValue()).longValue();
+                                    long jpegLength = jpegLenEntry != null ? ((Number) jpegLenEntry.getValue()).longValue() : -1;
+                                    if (jpegLength > 0 && jpegOffset + jpegLength <= stream.length()) {
+                                        // Verify first bytes are FFD8
+                                        stream.seek(jpegOffset);
+                                        stream.setByteOrder(ByteOrder.BIG_ENDIAN);
+                                        if (stream.readUnsignedShort() == JPEG.SOI) {
+                                            thumbnails.add(new EXIFThumbnailReader(thumbnailProgressDelegator, getThumbnailReader(), 0, thumbnails.size(), ifd1, stream));
+                                        }
+                                        // TODO: Simplify this warning fallback stuff...
+                                        else {
+                                            processWarningOccurred("EXIF IFD with empty or incomplete JPEG thumbnail");
+                                        }
+                                    }
+                                    else {
+                                        processWarningOccurred("EXIF IFD with empty or incomplete JPEG thumbnail");
+                                    }
                                 }
                                 else {
-                                    processWarningOccurred("EXIF IFD with empty (zero-length) thumbnail");
+                                    processWarningOccurred("EXIF IFD with JPEG thumbnail missing JPEGInterchangeFormat tag");
+                                }
+                            }
+                            else if (compression == 1) {
+                                Entry stripOffEntry = ifd1.getEntryById(TIFF.TAG_STRIP_OFFSETS);
+                                if (stripOffEntry != null) {
+                                    long stripOffset = ((Number) stripOffEntry.getValue()).longValue();
+
+                                    if (stripOffset < stream.length()) {
+                                        // TODO: Verify length of Exif thumbnail vs length of segment like in JPEG
+                                        //  ...but this requires so many extra values... Instead move this logic to the
+                                        //  EXIFThumbnailReader?
+                                        thumbnails.add(new EXIFThumbnailReader(thumbnailProgressDelegator, getThumbnailReader(), 0, thumbnails.size(), ifd1, stream));
+                                    }
+                                    else {
+                                        processWarningOccurred("EXIF IFD with empty or incomplete uncompressed thumbnail");
+                                    }
+                                }
+                                else {
+                                    processWarningOccurred("EXIF IFD with uncompressed thumbnail missing StripOffsets tag");
                                 }
                             }
                             else {
-                                processWarningOccurred("EXIF IFD with JPEG thumbnail missing JPEGInterchangeFormat tag");
+                                processWarningOccurred("EXIF IFD with unknown compression (expected 1 or 6): " + compression);
                             }
                         }
-                        else if (compression == 1) {
-                            if (ifd1.getEntryById(TIFF.TAG_STRIP_OFFSETS) != null) {
-                                thumbnails.add(new EXIFThumbnailReader(thumbnailProgressDelegator, getThumbnailReader(), 0, thumbnails.size(), ifd1, stream));
-                            }
-                            else {
-                                processWarningOccurred("EXIF IFD with uncompressed thumbnail missing StripOffsets tag");
-                            }
-                        }
-                        else {
-                            processWarningOccurred("EXIF IFD with unknown compression (expected 1 or 6): " + compression);
-                        }
+                    }
+                    catch (IIOException e) {
+                        processWarningOccurred("Exif chunk present, but can't be read: " + e.getMessage());
                     }
                 }
             }
@@ -1211,45 +1234,24 @@ public final class JPEGImageReader extends ImageReaderBase {
     public BufferedImage readThumbnail(int imageIndex, int thumbnailIndex) throws IOException {
         checkThumbnailBounds(imageIndex, thumbnailIndex);
 
-        return thumbnails.get(thumbnailIndex).read();
+//         processThumbnailStarted(imageIndex, thumbnailIndex);
+//         processThumbnailProgress(0f);
+
+        BufferedImage thumbnail = thumbnails.get(thumbnailIndex).read();;
+
+//         processThumbnailProgress(100f);
+//         processThumbnailComplete();
+
+        return thumbnail;
     }
 
     // Metadata
 
     @Override
     public IIOMetadata getImageMetadata(int imageIndex) throws IOException {
-        // checkBounds needed, as we catch the IndexOutOfBoundsException below.
-        checkBounds(imageIndex);
         initHeader(imageIndex);
 
-        IIOMetadata imageMetadata;
-
-        if (isLossless()) {
-            return new JPEGImage10Metadata(segments);
-        }
-        else {
-            try {
-                imageMetadata = delegate.getImageMetadata(0);
-            }
-            catch (IndexOutOfBoundsException knownIssue) {
-                // TMI-101: com.sun.imageio.plugins.jpeg.JPEGBuffer doesn't do proper sanity check of input data.
-                throw new IIOException("Corrupt JPEG data: Bad segment length", knownIssue);
-            }
-            catch (NegativeArraySizeException knownIssue) {
-                // Most likely from com.sun.imageio.plugins.jpeg.SOSMarkerSegment
-                throw new IIOException("Corrupt JPEG data: Bad component count", knownIssue);
-            }
-
-            if (imageMetadata != null && Arrays.asList(imageMetadata.getMetadataFormatNames()).contains(JPEGImage10MetadataCleaner.JAVAX_IMAGEIO_JPEG_IMAGE_1_0)) {
-                if (metadataCleaner == null) {
-                    metadataCleaner = new JPEGImage10MetadataCleaner(this);
-                }
-
-                return metadataCleaner.cleanMetadata(imageMetadata);
-            }
-        }
-
-        return imageMetadata;
+        return new JPEGImage10Metadata(segments, getSOF(), getJFIF(), getJFXX(), getEmbeddedICCProfile(true), getAdobeDCT(), getExif());
     }
 
     @Override
@@ -1417,7 +1419,7 @@ public final class JPEGImageReader extends ImageReaderBase {
             final String arg = args[argIdx];
 
             if (arg.charAt(0) == '-') {
-                if (arg.equals("-s") || arg.equals("--subsample") && args.length > argIdx) {
+                if (arg.equals("-s") || arg.equals("--subsample") && args.length > argIdx + 1) {
                     String[] sub = args[++argIdx].split(",");
 
                     try {
@@ -1436,7 +1438,7 @@ public final class JPEGImageReader extends ImageReaderBase {
                         System.err.println("Bad sub sampling (x,y): '" + args[argIdx] + "'");
                     }
                 }
-                else if (arg.equals("-r") || arg.equals("--roi") && args.length > argIdx) {
+                else if (arg.equals("-r") || arg.equals("--roi") && args.length > argIdx + 1) {
                     String[] region = args[++argIdx].split(",");
 
                     try {
