@@ -4,26 +4,28 @@
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
- *     * Redistributions of source code must retain the above copyright
- *       notice, this list of conditions and the following disclaimer.
- *     * Redistributions in binary form must reproduce the above copyright
- *       notice, this list of conditions and the following disclaimer in the
- *       documentation and/or other materials provided with the distribution.
- *     * Neither the name "TwelveMonkeys" nor the
- *       names of its contributors may be used to endorse or promote products
- *       derived from this software without specific prior written permission.
  *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
- * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
- * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
- * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
- * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
- * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
- * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
- * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
- * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
- * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
- * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ * * Redistributions of source code must retain the above copyright notice, this
+ *   list of conditions and the following disclaimer.
+ *
+ * * Redistributions in binary form must reproduce the above copyright notice,
+ *   this list of conditions and the following disclaimer in the documentation
+ *   and/or other materials provided with the distribution.
+ *
+ * * Neither the name of the copyright holder nor the names of its
+ *   contributors may be used to endorse or promote products derived from
+ *   this software without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+ * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
+ * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
+ * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
+ * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
+ * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
+ * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
+ * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
+ * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
 package com.twelvemonkeys.imageio.plugins.pcx;
@@ -54,7 +56,15 @@ import java.util.ArrayList;
 import java.util.Iterator;
 import java.util.List;
 
+/**
+ * ImageReader for ZSoft PC Paintbrush (PCX) format.
+ *
+ * @see <a href="http://www.drdobbs.com/pcx-graphics/184402396">PCX Graphics</a>
+ */
 public final class PCXImageReader extends ImageReaderBase {
+
+    final static boolean DEBUG = "true".equalsIgnoreCase(System.getProperty("com.twelvemonkeys.imageio.plugins.pcx.debug"));
+
     /** 8 bit ImageTypeSpecifer used for reading bitplane images. */
     private static final ImageTypeSpecifier GRAYSCALE = ImageTypeSpecifiers.createGrayscale(8, DataBuffer.TYPE_BYTE);
 
@@ -93,7 +103,7 @@ public final class PCXImageReader extends ImageReaderBase {
     public Iterator<ImageTypeSpecifier> getImageTypes(final int imageIndex) throws IOException {
         ImageTypeSpecifier rawType = getRawImageType(imageIndex);
 
-        List<ImageTypeSpecifier> specifiers = new ArrayList<ImageTypeSpecifier>();
+        List<ImageTypeSpecifier> specifiers = new ArrayList<>();
 
         // TODO: Implement
         specifiers.add(rawType);
@@ -107,27 +117,30 @@ public final class PCXImageReader extends ImageReaderBase {
         readHeader();
 
         int channels = header.getChannels();
-        int paletteInfo = header.getPaletteInfo();
-        ColorSpace cs = paletteInfo == PCX.PALETTEINFO_GRAY ? ColorSpace.getInstance(ColorSpace.CS_GRAY) : ColorSpace.getInstance(ColorSpace.CS_sRGB);
 
         switch (header.getBitsPerPixel()) {
             case 1:
             case 2:
             case 4:
+                // TODO: If there's a VGA palette here, use it?
+
                 return ImageTypeSpecifiers.createFromIndexColorModel(header.getEGAPalette());
             case 8:
-                // We may have IndexColorModel here for 1 channel images
-                if (channels == 1 && paletteInfo != PCX.PALETTEINFO_GRAY) {
+                if (channels == 1) {
+                    // We may have IndexColorModel here for 1 channel images
                     IndexColorModel palette = getVGAPalette();
-                    if (palette == null) {
-                        throw new IIOException("Expected VGA palette not found");
-                    }
 
-                    return ImageTypeSpecifiers.createFromIndexColorModel(palette);
+                    if (palette != null) {
+                        return ImageTypeSpecifiers.createFromIndexColorModel(palette);
+                    }
+                    else {
+                        // PCX Gray has 1 channel and no palette
+                        return ImageTypeSpecifiers.createGrayscale(8, DataBuffer.TYPE_BYTE);
+                    }
                 }
 
-                // PCX has 1 or 3 channels for 8 bit gray or 24 bit RGB, will be validated by ImageTypeSpecifier
-                return ImageTypeSpecifiers.createBanded(cs, createIndices(channels, 1), createIndices(channels, 0), DataBuffer.TYPE_BYTE, false, false);
+                // PCX RGB has channels for 24 bit RGB, will be validated by ImageTypeSpecifier
+                return ImageTypeSpecifiers.createBanded(ColorSpace.getInstance(ColorSpace.CS_sRGB), createIndices(channels, 1), createIndices(channels, 0), DataBuffer.TYPE_BYTE, channels == 4, false);
             case 24:
                 // Some sources says this is possible...
                 return ImageTypeSpecifiers.createFromBufferedImageType(BufferedImage.TYPE_3BYTE_BGR);
@@ -154,10 +167,6 @@ public final class PCXImageReader extends ImageReaderBase {
         Iterator<ImageTypeSpecifier> imageTypes = getImageTypes(imageIndex);
         ImageTypeSpecifier rawType = getRawImageType(imageIndex);
 
-        if (header.getPaletteInfo() != PCX.PALETTEINFO_COLOR && header.getPaletteInfo() != PCX.PALETTEINFO_GRAY) {
-            processWarningOccurred(String.format("Unsupported color mode: %d, colors may look incorrect", header.getPaletteInfo()));
-        }
-
         int width = getWidth(imageIndex);
         int height = getHeight(imageIndex);
 
@@ -174,8 +183,8 @@ public final class PCXImageReader extends ImageReaderBase {
 
         // Wrap input (COMPRESSION_RLE is really the only value allowed)
         DataInput input = compression == PCX.COMPRESSION_RLE
-                ? new DataInputStream(new DecoderStream(IIOUtil.createStreamAdapter(imageInput), new RLEDecoder()))
-                : imageInput;
+                          ? new DataInputStream(new DecoderStream(IIOUtil.createStreamAdapter(imageInput), new RLEDecoder()))
+                          : imageInput;
 
         int xSub = param != null ? param.getSourceXSubsampling() : 1;
         int ySub = param != null ? param.getSourceYSubsampling() : 1;
@@ -185,16 +194,16 @@ public final class PCXImageReader extends ImageReaderBase {
         if (rawType.getColorModel() instanceof IndexColorModel && header.getChannels() > 1) {
             // Bit planes!
             // Create raster from a default 8 bit layout
-            WritableRaster rowRaster = GRAYSCALE.createBufferedImage(header.getWidth(), 1).getRaster();
+            int planeWidth = header.getBytesPerLine();
+            int rowWidth = planeWidth * 8; // bitsPerPixel == 1
+            WritableRaster rowRaster = GRAYSCALE.createBufferedImage(rowWidth, 1).getRaster();
 
             // Clip to source region
             Raster clippedRow = clipRowToRect(rowRaster, srcRegion,
-                                              param != null ? param.getSourceBands() : null,
-                                              param != null ? param.getSourceXSubsampling() : 1);
-
-            int planeWidth = header.getBytesPerLine();
-            byte[] planeData = new byte[planeWidth * 8];
+                    param != null ? param.getSourceBands() : null,
+                    param != null ? param.getSourceXSubsampling() : 1);
 
+            byte[] planeData = new byte[rowWidth];
             byte[] rowDataByte = ((DataBufferByte) rowRaster.getDataBuffer()).getData();
 
             for (int y = 0; y < height; y++) {
@@ -257,8 +266,8 @@ public final class PCXImageReader extends ImageReaderBase {
 
             // Clip to source region
             Raster clippedRow = clipRowToRect(rowRaster, srcRegion,
-                                              param != null ? param.getSourceBands() : null,
-                                              param != null ? param.getSourceXSubsampling() : 1);
+                    param != null ? param.getSourceBands() : null,
+                    param != null ? param.getSourceXSubsampling() : 1);
 
             for (int y = 0; y < height; y++) {
                 for (int c = 0; c < header.getChannels(); c++) {
@@ -351,13 +360,19 @@ public final class PCXImageReader extends ImageReaderBase {
         if (header == null) {
             imageInput.setByteOrder(ByteOrder.LITTLE_ENDIAN);
             header = PCXHeader.read(imageInput);
+
+            if (DEBUG) {
+                System.err.println("header: " + header);
+            }
+
             imageInput.flushBefore(imageInput.getStreamPosition());
         }
 
         imageInput.seek(imageInput.getFlushedPosition());
     }
 
-    @Override public IIOMetadata getImageMetadata(final int imageIndex) throws IOException {
+    @Override
+    public IIOMetadata getImageMetadata(final int imageIndex) throws IOException {
         checkBounds(imageIndex);
         readHeader();
 
@@ -371,29 +386,27 @@ public final class PCXImageReader extends ImageReaderBase {
             // Mark palette as read, to avoid further attempts
             readPalette = true;
 
-            // Wee can't simply skip to an offset, as the RLE compression makes the file size unpredictable
-            skipToEOF(imageInput);
+            if (header.getVersion() >= PCX.VERSION_3 || header.getVersion() == PCX.VERSION_2_8_PALETTE) {
+                // We can't simply skip to an offset, as the RLE compression makes the file size unpredictable
+                skipToEOF(imageInput);
 
-            // Seek backwards from EOF
-            long paletteStart = imageInput.getStreamPosition() - 769;
-            if (paletteStart <= imageInput.getFlushedPosition()) {
-                return null;
+                int paletteSize = 256 * 3; // 256 * 3 for RGB
+
+                // Seek backwards from EOF
+                long paletteStart = imageInput.getStreamPosition() - paletteSize - 1;
+                if (paletteStart > imageInput.getFlushedPosition()) {
+                    imageInput.seek(paletteStart);
+
+                    byte val = imageInput.readByte();
+
+                    if (val == PCX.VGA_PALETTE_MAGIC) {
+                        byte[] palette = new byte[paletteSize];
+                        imageInput.readFully(palette);
+
+                        vgaPalette = new IndexColorModel(8, 256, palette, 0, false);
+                    }
+                }
             }
-
-            imageInput.seek(paletteStart);
-
-            byte val = imageInput.readByte();
-
-            if (val == PCX.VGA_PALETTE_MAGIC) {
-                byte[] palette = new byte[768]; // 256 * 3 for RGB
-                imageInput.readFully(palette);
-
-                vgaPalette = new IndexColorModel(8, 256, palette, 0, false);
-
-                return vgaPalette;
-            }
-
-            return null;
         }
 
         return vgaPalette;
@@ -413,7 +426,7 @@ public final class PCXImageReader extends ImageReaderBase {
             long pos = stream.getStreamPosition();
 
             // ...skip 1k blocks until we're passed EOF...
-            while (stream.skipBytes(1024l) > 0) {
+            while (stream.skipBytes(1024L) > 0) {
                 if (stream.read() == -1) {
                     break;
                 }
